/**
 * @file
 * 3D Image behaviors.
 */

(function ($, Drupal) {

  'use strict';

  /**
   * Behavior description.
   */
  Drupal.behaviors.threeDImage = {
    attach: function (context, settings) {
      $(document, context).once('three_d_image_field').find('.three-d-image').each((i, e) => {
        /** @type {HTMLCanvasElement} */
        const canvas = e;
        const gl = canvas.getContext("webgl");
        if (!gl) {
          return;
        }

        let originalImage = { width: 1, height: 1 };
        const originalTexture = twgl.createTexture(gl, {
          src: e.dataset.img,
          crossOrigin: '',
        }, (err, texture, source) => {
          originalImage = source;
        });

        const mapTexture = twgl.createTexture(gl, {
          src: e.dataset.depthMap, crossOrigin: '',
        });

        const programInfo = twgl.createProgramInfo(gl, ["vs", "fs"]);
        const bufferInfo = twgl.primitives.createXYQuadBufferInfo(gl);
        const mouse = [0, 0];

        document.addEventListener('mousemove', (event) => {
          mouse[0] = (event.clientX / gl.canvas.clientWidth * 2 - 1) * -0.01;
          mouse[1] = (event.clientY / gl.canvas.clientHeight * 2 - 1) * -0.01;
        });

        document.addEventListener('touchmove', (event) => {
          mouse[0] = (event.touches[0].clientX / gl.canvas.clientWidth * 2 - 1) * -0.01;
          mouse[1] = (event.touches[0].clientY / gl.canvas.clientHeight * 2 - 1) * -0.01;
        });

        document.addEventListener('touchend', (event) => {
          mouse[0] = 0;
          mouse[1] = 0;
        });

        var nMouse = [0, 0];
        var oMouse = [0, 0];

        requestAnimationFrame(render);

        function render() {

          twgl.resizeCanvasToDisplaySize(gl.canvas);

          gl.viewport(0, 0, gl.canvas.width, gl.canvas.height);

          gl.clearColor(0, 0, 0, 0);
          gl.clear(gl.COLOR_BUFFER_BIT);

          gl.useProgram(programInfo.program);

          twgl.setBuffersAndAttributes(gl, programInfo, bufferInfo);

          const canvasAspect = gl.canvas.clientWidth / gl.canvas.clientHeight;
          const imageAspect = originalImage.width / originalImage.height;
          const mat = m3.scaling(imageAspect / canvasAspect, -1);

          nMouse[0] += (mouse[0] - nMouse[0]) * 0.05;
          nMouse[1] += (mouse[1] - nMouse[1]) * 0.05;

          twgl.setUniforms(programInfo, {
            u_matrix: mat,
            u_originalImage: originalTexture,
            u_mapImage: mapTexture,
            u_mouse: nMouse,
          });

          twgl.drawBufferInfo(gl, bufferInfo);

          requestAnimationFrame(render);
        }

      });


    }
  };

}(jQuery, Drupal));
